<?php

namespace Drupal\ckeditor5_audio_plugin\Controller;

use Drupal\Core\Controller\ControllerBase;
use Drupal\Core\File\FileSystemInterface;
use Drupal\Core\File\FileUrlGeneratorInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Symfony\Component\HttpFoundation\JsonResponse;
use Symfony\Component\HttpFoundation\Request;

/**
 * Returns response for CKEditor 5 audio upload adapter.
 *
 * @internal
 *   Controller classes are internal.
 */
class AudioUploadController extends ControllerBase {

  /**
   * The file system service.
   */
  protected FileSystemInterface $fileSystem;

  /**
   * The The file url generator.
   */
  protected FileUrlGeneratorInterface $fileUrlGenerator;

  /**
   * Constructs a new CKEditor5ImageController.
   *
   * @param \Drupal\Core\File\FileSystemInterface $fileSystem
   *   The file system service.
   * @param \Drupal\Core\File\FileUrlGeneratorInterface $fileUrlGenerator
   *   The file url generator.
   */
  public function __construct(
    FileSystemInterface $fileSystem,
    FileUrlGeneratorInterface $fileUrlGenerator
  ) {
    $this->fileSystem = $fileSystem;
    $this->fileUrlGenerator = $fileUrlGenerator;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    return new static(
      $container->get('file_system'),
      $container->get('file.file_url_generator')
    );
  }

  /**
   * Uploads and saves an audio from a CKEditor 5 POST.
   *
   * @param \Symfony\Component\HttpFoundation\Request $request
   *   The current request object.
   *
   * @return \Symfony\Component\HttpFoundation\JsonResponse
   *   A JSON object including the file URL with status 200
   *   or error with status 400.
   */
  public function upload(Request $request) {
    $file = $request->files->get('file');
    $directory = $request->get('directory');
    $max_size = $request->get('max_size');

    if (!$file) {
      return new JsonResponse(['error' => 'No file uploaded'], 400);
    }

    // Validate max file size.
    if ($max_size && $file->getSize() > (int) $max_size) {
      return new JsonResponse(['error' => 'File exceeds maximum size'], 400);
    }

    // Validate directory input and ensure it's safe.
    $directory = empty($directory) ? 'public://' : sprintf('public://%s', $directory);

    if (!$this->fileSystem->prepareDirectory($directory, FileSystemInterface::CREATE_DIRECTORY)) {
      return new JsonResponse(['error' => 'Can not upload file'], 400);
    };
    $destination = $directory . '/' . $file->getClientOriginalName();
    $this->fileSystem->move($file->getRealPath(), $destination);

    $fileUrl = $this->fileUrlGenerator->generateAbsoluteString($destination);
    return new JsonResponse(['url' => $fileUrl]);
  }

}
